package com.logicbig.example;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Repository;

import javax.annotation.PostConstruct;
import javax.sql.DataSource;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;
import java.sql.*;
import java.util.List;
import java.util.Scanner;

@Repository
public class ReportDao implements Dao<Report> {
    @Autowired
    private DataSource dataSource;
    private JdbcTemplate jdbcTemplate;

    @PostConstruct
    private void postConstruct() {
        jdbcTemplate = new JdbcTemplate(dataSource);
    }

    @Override
    public long save(Report report) {
        String sql = "insert into REPORT (NAME, CONTENT, IMAGE) values (?, ?, ?)";
        KeyHolder holder = new GeneratedKeyHolder();
        jdbcTemplate.update(new PreparedStatementCreator() {
            @Override
            public PreparedStatement createPreparedStatement(Connection connection)
                    throws SQLException {
                PreparedStatement ps = connection.prepareStatement(sql.toString(),
                        Statement.RETURN_GENERATED_KEYS);
                ps.setString(1, report.getName());
                Reader reader = new StringReader(report.getContent());
                ps.setClob(2, reader);
                ByteArrayInputStream inputStream = new ByteArrayInputStream(report.getImage());
                ps.setBlob(3, inputStream);
                return ps;
            }
        }, holder);
        Number key = holder.getKey();
        if (key != null) {
            return key.longValue();
        }
        throw new RuntimeException("No generated primary key returned.");
    }

    @Override
    public Report load(long id) {
        List<Report> persons = jdbcTemplate.query("select * from Report where id =?",
                new Object[]{id}, (resultSet, i) -> {
                    return toReport(resultSet);
                });
        if (persons.size() == 1) {
            return persons.get(0);
        }
        throw new RuntimeException("No item found for id: " + id);
    }

    @Override
    public int delete(long id) {
        return 0; //todo
    }

    @Override
    public int update(Report customer) {
        return 0;//todo
    }

    @Override
    public List<Report> loadAll() {
        return jdbcTemplate.query("select * from REPORT", (resultSet, i) -> {
            return toReport(resultSet);
        });
    }

    private Report toReport(ResultSet resultSet) throws SQLException {
        Report report = new Report();
        report.setId(resultSet.getLong("ID"));
        report.setName(resultSet.getString("NAME"));
        InputStream contentStream = resultSet.getClob("CONTENT")
                                             .getAsciiStream();
        String content =
                new Scanner(contentStream, "UTF-8").useDelimiter("\\A").next();
        report.setContent(content);
        Blob blob = resultSet.getBlob("IMAGE");
        byte[] bytes = blob.getBytes(1, (int) blob.length());
        report.setImage(bytes);
        return report;
    }
}